#include <stdio.h>
#include <math.h>
#include <sys/time.h>
#include <X11/Xlib.h>

#define EVENT_MASKS (PointerMotionMask)

Display *dpy;
Window   rootwin;
int cenx, ceny, rootwid, roothgt;
int getevents = 0;

void init(dispname)
char *dispname;
{
    int      scn;
    XSetWindowAttributes attr;
    XWindowAttributes rattr;
    XGCValues gcvalues;

    /*
     * connect to the X server.  uses the server specified in the
     * DISPLAY environment variable
     */
    dpy = XOpenDisplay(dispname);
    if ((Display *) NULL == dpy) {
	fprintf(stderr, "xnuisance:  could not open display.\n");
	exit(-1);
    }

    /*
     * begin to create a window
     */
    scn = DefaultScreen(dpy);
    rootwin = RootWindow(dpy, scn);
    XGetWindowAttributes(dpy, rootwin, &rattr);
    rootwid = rattr.width;
    roothgt = rattr.height;
    cenx = rattr.width/2;
    ceny = rattr.height/2;

    if (getevents) {
	attr.event_mask = EVENT_MASKS;
	XChangeWindowAttributes(dpy, rootwin, CWEventMask, &attr);
    }
}

void usage()
{
    printf("usage: xnuisance [-jitterbug [speed]] [-sidehillhoofer [speed]] [-hippopotamus [weight]]\n");
    exit(-1);
}

main (argc, argv)
int argc;
char *argv[];
{
    Bool   done = False;
    XEvent event;
    char   c;
    int xoff, yoff, xold, yold;
    int xlast, ylast;
    int ix, temp;
    unsigned int maskr;
    Window rr, cr;
    struct timeval stv;
    int veryfirst;
    double hippox=0.0, hippoy=0.0, hippodist;

    int jitter = 0;
    int hoofer = 0;
    int hippo = 0;

    ix = 1;
    while (ix<argc) {
	if (!strcmp(argv[ix], "-jitterbug")) {
	    jitter = 6;
	    ix++;
	    if (ix<argc && isdigit(argv[ix][0])) {
		jitter = atol(argv[ix]);
		if (jitter>100)
		    jitter = 100;
		if (jitter<0)
		    jitter = 0;
		ix++;
	    }
	}
	else if (!strcmp(argv[ix], "-sidehillhoofer")) {
	    hoofer = 5;
	    ix++;
	    if (ix<argc && isdigit(argv[ix][0])) {
		hoofer = atol(argv[ix]);
		if (hoofer>20)
		    hoofer = 20;
		if (hoofer<0)
		    hoofer = 0;
		ix++;
	    }
	    hoofer *= 3;
	}
	else if (!strcmp(argv[ix], "-hippopotamus")) {
	    hippo = 12;
	    ix++;
	    if (ix<argc && isdigit(argv[ix][0])) {
		hippo = atol(argv[ix]);
		if (hippo>30)
		    hippo = 30;
		if (hippo<0)
		    hippo = 0;
		ix++;
	    }
	}
	else
	    usage();
    }

    if (hoofer==0 && jitter==0 && hippo==0) {
	printf("xnuisance: you must select at least one option.\n");
	usage();
    }

    if (hippo) {
	getevents = 1;
	hippodist = 0.01 * hippo;
    };
	
    init(NULL);

    veryfirst = 1;
    while (!done) {
	while (getevents && XCheckWindowEvent(dpy, rootwin, EVENT_MASKS, &event)) {
	    switch (event.type) {
		case MotionNotify: {
		    XMotionEvent *movent = (XMotionEvent *) &event;
		    if (veryfirst)
			veryfirst = 0;
		    else {
			if (xoff == movent->x_root && yoff == movent->y_root) {
			}
			else {
			    xold = movent->x_root - xlast;
			    yold = movent->y_root - ylast;
			    hippox += (double)xold;
			    hippoy += (double)yold;
			}
		    }
		    xlast = movent->x_root;
		    ylast = movent->y_root;
		    }
		    break;
		default:
		    printf("Other event %d\n", event.type);
		    break;
	    }
	}
	if (XQueryPointer(dpy, rootwin, &rr, &cr, &xold, &yold, &temp, &temp, &maskr)) {
	    double vx, vy, dd;

	    if (!hoofer) {
		xoff = xold;
		yoff = yold;
	    }
	    else {
		dd = sqrt((double)((cenx - xold) * (cenx - xold) + (ceny - yold) * (ceny - yold)));
		vx = (double)(cenx - xold) / dd;
		vy = (double)(ceny - yold) / dd;
		yoff = yold + (int)((vx + 0.02*vy) * (double)hoofer);
		xoff = xold + (int)((-vy + 0.02*vx) * (double)hoofer);
	    }
	    if (hippo) {
		xoff += (int)(hippox*hippodist + 0.5);
		yoff += (int)(hippoy*hippodist + 0.5);
		hippox *= 0.8;
		hippoy *= 0.8;
	    }
	    if (jitter) {
		xoff += (random()%(jitter*2+1) - jitter);
		yoff += (random()%(jitter*2+1) - jitter);
	    }
	    if (xoff<0) xoff = 0;
	    else if (xoff>=rootwid-1) xoff = rootwid-2;
	    if (yoff<0) yoff = 0;
	    else if (yoff>=roothgt-1) yoff = roothgt-2;

	    if (xoff!=xold || yoff!=yold)
		XWarpPointer(dpy, None, rootwin, 0, 0, 0, 0, xoff, yoff);
	}
	XSync(dpy, 0);
	stv.tv_sec = 0;
	stv.tv_usec = 50000;
	(void)select(0, 0, 0, 0, &stv);
    }

    XCloseDisplay(dpy);
}
